//--------------------------------------------------------------------------------------------------------------
// The tracepoint provider header file consist of 3 sections below:
//   SECTION 1   (this section can be automatically filled out by a helper script)
//   SECTION 2   (this section has to be manually filled out by the instrumenter)
//   SECTION 3   (this section can be automatically filled out by a helper script)
//
// In case the instrumenter does not want to worry about SECION-1 and SECTION-3,
// the helper script "lttng-gen-tp" can be used . This helper script is a 3rd party
// product (created by Lttng) and is not supported by the Trace component design
// team yet. However, it is included in the TraceEA SDK in case the instrumenter
// wants to use it.
// It should be found under /<where TraceEA SDK is installed>/usr/bin/
//
// 1)_ The instrumenter needs to fill out SECTION-2 and stored it in a file with
//     filename = "the same file name as the tracepoint provider header file" with extension ".template"
//     Example: The tracepoint provider header file is: "com_ericsson_applicationx.h"
//              Then the template file should be called: "com_ericsson_applicationx.template"
//
// 2)_ ./lttng-gen-tp com_ericsson_applicationx.template -o com_ericsson_applicationx.h
//
// 3)_ The output file: "com_ericsson_applicationx.h" will have all 3 sections filled out
//     where section 2 is the exact contents of the template file in step-2.
// 
// 4)_ WARNING: At the time this document is written, there is no error checking in the 
//              helper script. Therefore, the instrumenter has to make sure the output from
//              this tool complies with the guideline outined in the Trace Programmer Guide.
//              
//--------------------------------------------------------------------------------------------------------------


//------------------------------------------------------
// SECTION 1
//------------------------------------------------------
#undef TRACEPOINT_PROVIDER
#define TRACEPOINT_PROVIDER com_ericsson_applicationx
                            //-- the above tracepoint provider name represents the domain part of the event
#undef TRACEPOINT_INCLUDE_FILE
#define TRACEPOINT_INCLUDE_FILE ./com_ericsson_applicationx.h
                            //-- The name of the tracepoint provider header file is recommended to have the 
                            //-- same name as the tracepoint provider itself.
                            //-- in this case, the tracepoint provider is "com_ericsson_applicationx"
                            //-- therefore, the name of this tracepoint provider header file should be:
                            //-- "com_ericsson_applicationx.h"
                            //-- This will prevent name clashing later on.
                             
#ifdef __cplusplus
extern "C" {
#endif

#if !defined(_COM_ERICSSON_APPLICATIONX_H) || defined(TRACEPOINT_HEADER_MULTI_READ)
#define _COM_ERICSSON_APPLICATIONX_H
                            //-- the above definition should be "_" + "the name of the tracepoint provider in capital"
                            //   + "_H" .
                            //   In this case: _COM_ERICSSON_APPLICATIONX_H

#include <lttng/tracepoint.h>
//------------------------------------------------------
// End of SECTION 1
//------------------------------------------------------


//----------------------------------------------------------------------------------------
// SECTION 2: (MANUAL SECTION)
//
// The following section has to be manually filled in by the instrumenter
// to specify which event name to be used and how the parameters in the payload 
// need to be output in the log, and also the corresponding loglevel of each
// tracepoint.
//
//----------------------------------------------------------------------------------------

//--- tracepoint 1 (com_ericsson_applicationx:MyEventName1)
//    This tracepoint illustrates that the parameter passed in tracepoint statement
//    are passed by value. In this case, parameter 'i' is passed in the tracepoint()
//    statement from the main program, in TP_ARGS() it is received with the name
//    LoopCounter_i. In TP_FIELDS, it is printed out with the name MyLoopCounter_i.
//    This just shows that there are flexibilty to modify the parameter name 
//    at printout of the event.
//    However, it is strongly recommended to use the same parameter name 
//    as passed in the tracepoint() statement. This is highlighted in the
//    second integer parameter "i".

TRACEPOINT_EVENT(com_ericsson_applicationx, MyEventName1,
                 TP_ARGS(int, LoopCounter_i,
                         int, MyInt
                          ),
                 TP_FIELDS(
                            ctf_integer(int, MyLoopCounter_i, LoopCounter_i)
                            ctf_integer(int, MyInt, MyInt)
                          )
)

TRACEPOINT_LOGLEVEL(com_ericsson_applicationx, MyEventName1, TRACE_WARNING)

//--- tracepoint 3 (com_ericsson_applicationx:MyEventName2)
//    This tracepoint illustrates that when "ctf_string" macro is going
//    to be used, there is no need to pass the string length parameter of
//    the string in TP_ARGS().
TRACEPOINT_EVENT(com_ericsson_applicationx, MyEventName2,
                 TP_ARGS( const char *, MyString
                          ),
                 TP_FIELDS(ctf_string(string, MyString)
                          )
)
TRACEPOINT_LOGLEVEL(com_ericsson_applicationx, MyEventName2, TRACE_WARNING)

//--- tracepoint 3 (com_ericsson_applicationx:MyEventName3)
//    This tracepoint illustrates other ctf macro printout which are supported
//    by Lttng. They are used here just as an example.
//    NOTE: This tracepoint illustrates also that more arguments can be passed into 
//          TP_FIELDS() macro than the number of arguments passed in TP_ARGS() macro.
TRACEPOINT_EVENT(com_ericsson_applicationx, MyEventName3,
                 TP_ARGS( char *, array_text, 
                          size_t, array_text_length,
                          int, MyNewLength,
                          int, MyOverflowLength                    
                          ),
                 TP_FIELDS(ctf_array(char, array, array_text, 3) // output in trace log
                           ctf_array(char, array2, array_text, 1)
                           ctf_array_text(char, arrayText, array_text, 2)
                           ctf_array_text(char, arrayText2, array_text, 20) // Note 20 > array_length
                           ctf_sequence(char, Sequence, array_text, size_t,array_text_length)
                           ctf_sequence_text(char, sequenceText, array_text, size_t,array_text_length)
                           ctf_sequence_text(char, sequenceText2, array_text, size_t,MyNewLength)
                           ctf_sequence_text(char, sequenceText3, array_text, size_t,MyOverflowLength)
                           ctf_string(String, array_text)
                          )
)
TRACEPOINT_LOGLEVEL(com_ericsson_applicationx, MyEventName3, TRACE_WARNING)

//----------------------------------------------------------------------------------------
// End of SECTION 2
//----------------------------------------------------------------------------------------

//------------------------------------------------------
// SECTION 3
//------------------------------------------------------

#endif /* _COM_ERICSSON_APPLICATIONX_H */

/* This part must be outside protection */
#include <lttng/tracepoint-event.h>

#ifdef __cplusplus 
}
#endif
//------------------------------------------------------
// END of SECTION 3
//------------------------------------------------------
