#include <stdlib.h>
#include <sys/time.h>
#include <iostream>
#include <string>
#include <sstream>
#include <ctime>
#include <unistd.h>
#include <string.h>

using namespace std;

#define TRACEPOINT_DEFINE
#define TRACEPOINT_PROBE_DYNAMIC_LINKAGE
#include "TestApp.h"
#include "com_ericsson_cba.h"
#include "com_ericsson_cba_trace.h"
#include "com_ericsson_cba_trace_testapp.h"
#include "com_ericsson_cba_trace_testapp_lowtraf.h"
#include "com_ericsson_cba_trace_testapp_hightraf.h"

//#include "ust_tests_demo.h"

/**
 * Allocate memory and return a pointer to the subscriber structure
 *
 **/
p_subscriber get_p_subscriber(){
    p_subscriber temp = (p_subscriber)malloc(sizeof(p_subscriber *));
    return temp;
}

string GetTime ()
{
    struct tm *ttime;
    time_t rawtime;
    char tempT[20];
    string timeString;

    time( &rawtime );
    ttime = localtime(&rawtime);
    strftime(tempT,60,"%H:%M:%S",ttime); // the time is now in time_string
    timeString.assign(tempT,9);          // convert from char* to string
    return(timeString);
}

unsigned long long ourTime;
unsigned long long nowTime;


void mySleep(unsigned int ns)
{
    static unsigned loop_count = 0;
    ourTime += ns;
    ++loop_count;
    if (ourTime <= nowTime)
        return;
    if (loop_count > 10) // 10 is arbitrary
    {
        struct timespec now;
        clock_gettime(CLOCK_MONOTONIC, &now);
        nowTime = now.tv_sec*1000000000UL + now.tv_nsec;
        loop_count = 0;
        if (ourTime <= nowTime)
            return;
    }

    do {
        usleep( (ourTime - nowTime + 999)/1000 );

        struct timespec now;
        clock_gettime(CLOCK_MONOTONIC, &now);
        nowTime = now.tv_sec*1000000000UL + now.tv_nsec;
    } while( ourTime > nowTime);
    loop_count = 0;
}

int main (int argc, char **argv)
{
    char my_Text[10]="Hello";
    int OnePs, TenPs, HundredPs, KiloPs;  // for loop counters (x per second)
    int TenKiloPs;
    //int HundredKiloPs;

    int LoopTime = 1;                     // By default this program will run 1 sec
    bool PrintFlag = false;               // By default, timestamp printout per sec is not enabled.
    string my_sTime, my_sProgramInfo;
    string stringAllSigns = "~`!@#$%^&*()-_=+\\|{}[]'\":;?/>.<,";
    
    float Pi_Value = 3.14;
    long  My_LongArray[] = {1111, 2222, 3333, 4444, 5555};

    struct timespec now;
    clock_gettime(CLOCK_MONOTONIC, &now);
    nowTime = now.tv_sec*1000000000 + now.tv_nsec;
    ourTime = nowTime - nowTime % 100000;

    //* data to be used for conditioning tracing (filter):
    const string unit_string[10]={"unit_0", "unit_1", "unit_2", "unit_3", "unit_4", \
        "unit_5", "unit_6", "unit_7", "unit_8", "unit_9"};
    const float FLOAT_NUMBER = 0.1234;
    float unit_float;

    //* end of data for conditioning tracing

    my_sTime = GetTime();

    if ((argc < 2) || (argc > 3))
    {
        cout << " \n\n Usage: TestApp_type1 <num_of_sec_to_run> [np] \n\n";
        cout << "        ex: TestApp_type1 5000    (will run for 5000 sec, time stamp is printed out every sec)\n";
        cout << "            TestApp_type1 5000 np (will run fir 5000 sec, suppress printing of time stamp )\n\n";
        return 1;
    }
    else if (argc == 2)
    {
        LoopTime = atoi(argv[1]);
        PrintFlag = true;
    }
    else
    {
        LoopTime = atoi(argv[1]);
        PrintFlag = false;
    }

    //**** poppulate the subsriber structure to be passed in tracepoint:
    p_subscriber personA = get_p_subscriber();
    personA->name = "Micky Mouse";
    personA->imsi = "123455143457900";
    personA->balance = 50.25;
    personA->secondRemaining = 3600;

    if (ourTime % 1000000000 < 200000000) // Make sure our bias can be applied without skipping a seccond
        mySleep(100000);

    int IncrementMe = 1;
    my_sProgramInfo = "TestApp_type1";
    OnePs=0;
    TenPs     = (ourTime % 1000000000) / 100000000;
    HundredPs = (ourTime % 100000000)  / 10000000;
    KiloPs    = (ourTime % 10000000)   / 1000000;
    TenKiloPs = (ourTime % 1000000)    / 100000;
    //HundredKiloPs = (ourTime % 100000) / 10000;
    TenPs = (TenPs - 1) % 10; // Bias the time so that we allways call time() .1s after the second as it seems to not be synced


    do {
        if (PrintFlag)
        {
            cout << "com_ericsson_cba_trace_testapp_lowtraf     1/s: this is the " << OnePs << "th time, at: ";
            cout << my_sTime.c_str() << ", " << ourTime << "----------" << endl;
        }
        tracepoint(com_ericsson_cba,               cba_domain_event,     my_sTime.c_str(), OnePs, my_sProgramInfo.c_str(), IncrementMe);
        tracepoint(com_ericsson_cba_trace,         trace_domain_event,   my_sTime.c_str(), OnePs, my_sProgramInfo.c_str(), My_LongArray, sizeof(My_LongArray)/sizeof(long), my_Text, strlen(my_Text) );
        tracepoint(com_ericsson_cba_trace_testapp, testapp_domain_event, my_sTime.c_str(), OnePs, my_sProgramInfo.c_str(), Pi_Value);

        tracepoint(com_ericsson_cba_trace_testapp_lowtraf, OnePerSecA, my_sTime.c_str(), OnePs);
        tracepoint(com_ericsson_cba_trace_testapp_lowtraf, OnePerSecB, my_sTime.c_str());

        //***** attempt to pass a pointer of a structure to the tracepoint()
        //
        tracepoint(com_ericsson_cba_trace_testapp, pointer_to_structure_in_tracepoint, my_sTime.c_str(), personA);


        do {  //----- Ten per sec
            //cout << "com_ericsson_cba_trace_testapp_lowtraf       10/s: this is the " << TenPs << "th time within 1 sec. \n";

            unit_float = TenPs + FLOAT_NUMBER;
            tracepoint(com_ericsson_cba_trace_testapp_lowtraf, TenPerSecA, my_sTime.c_str(), OnePs, TenPs, unit_float,unit_string[TenPs].c_str(),stringAllSigns.c_str());
            tracepoint(com_ericsson_cba_trace_testapp_lowtraf, TenPerSecB, my_sTime.c_str());

            do { //----- Hundred per sec
                //cout << "com_ericsson_cba_trace_testapp_hightraf          100/s: this is the " << (TenPs*10) + HundredPs << "th time within 1 sec. \n";
                unit_float = HundredPs + FLOAT_NUMBER;
                tracepoint(com_ericsson_cba_trace_testapp_hightraf, HundredPerSec, my_sTime.c_str(), OnePs, TenPs, HundredPs, unit_float, unit_string[HundredPs].c_str());

                do { //----- Kilo per sec
                    //cout << "com_ericsson_cba_trace_testapp_hightraf             1000/s this is the " << (TenPs*100)+(HundredPs*10) + KiloPs +1 << "th time within 1 sec. \n";
                    unit_float = KiloPs + FLOAT_NUMBER;
                    tracepoint(com_ericsson_cba_trace_testapp_hightraf, KiloPerSec, my_sTime.c_str(), OnePs, TenPs, HundredPs, KiloPs, unit_float, unit_string[KiloPs].c_str());


                    do { //----- 10Kilo per sec
                        //cout << "time = " << my_sTime.c_str() << " OnePs=" << OnePs << " TenPs=" << TenPs << " HundredPs=" << HundredPs << " KiloPs=" << KiloPs << " TenKiloPs=" << TenKiloPs << endl;
                        unit_float = TenKiloPs + FLOAT_NUMBER;
                        tracepoint(com_ericsson_cba_trace_testapp_hightraf, TenKiloPerSec, my_sTime.c_str(), OnePs, TenPs, HundredPs, KiloPs, TenKiloPs, unit_float, unit_string[TenKiloPs].c_str());
#if 0
                        do { //----- 100Kilo per sec
                            //cout << "com_ericsson_cba_trace_testapp_hightraf       100 x 1000/s this is the " << (TenPs*10000)+(HundredPs*1000) + (KiloPs*100) + (TenKiloPs*10) + HundredKiloPs +1 << "th time within 1 sec. \n";
                            //cout << "time = " << my_sTime.c_str() << " OnePs=" << OnePs << " TenPs=" << TenPs << " HundredPs=" << HundredPs << " KiloPs=" << KiloPs << " TenKiloPs=" << TenKiloPs << " HundredKiloPs=" << HundredKiloPs << endl;
                            unit_float = HundredKiloPs + FLOAT_NUMBER;
                            tracepoint(com_ericsson_cba_trace_testapp_hightraf, HundredKiloPerSec, my_sTime.c_str(), OnePs, TenPs, HundredPs, KiloPs, TenKiloPs, HundredKiloPs, unit_float, unit_string[HundredKiloPs].c_str());
                            mySleep(10000);
                        } while ( ++HundredKiloPs < 10);
                        HundredKiloPs = 0;
#else
                        mySleep(100000);
#endif
                    } while ( ++TenKiloPs < 10);
                    TenKiloPs = 0;
                } while ( ++KiloPs < 10);
                KiloPs = 0;
            } while ( ++HundredPs < 10);
            HundredPs = 0;
        } while ( ++TenPs < 10);
        TenPs=0;

        my_sTime = GetTime();

    } while (++OnePs < LoopTime);

    cout << my_sTime.c_str() << "end of program \n";

    return 0;
}
